﻿/*
 * author:Joey Zhao
 * date:2010-10-8
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Data;
using Joey.DbOperator;
using GuYu.Todo.Entities;

namespace GuYu.Todo.Forms.BusinessNotify
{
    public class NotifyList
    {
        private DataBase _db = new DataBase();
        private List<NotifyEntity> _list;
        private System.Windows.Forms.Timer _timer;
        private static NotifyList _notifyList;
        private DateTime _today;
        private string _exeFilePath;

        private NotifyList()
        {
            _today = DateTime.Now;
            _list = new List<NotifyEntity>();
            Thread thread = new Thread(this.LoadData);
            thread.Start();
            _timer = new System.Windows.Forms.Timer();
            _timer.Interval = 1000 * 60;
            _timer.Tick += new EventHandler(_timer_Tick);
        }

        //定时扫描提醒列表
        private void _timer_Tick(object sender, EventArgs e)
        {
            DateTime now = DateTime.Now;
            if (now.DayOfYear != _today.DayOfYear)//0:00后重新加载一次数据
            {
                _today = now;
                this.LoadData();
            }
            foreach (var item in _list)
            {
                if (item.NotifyTime == now.Hour * 60 + now.Minute)
                {
                    this.Notify(item);
                }
            }
        }

        //提醒
        private void Notify(NotifyEntity item)
        {
            if (item.NotifyContent.Is_Exe_Notify == 1)
            {
                _exeFilePath = item.NotifyContent.Exe_Path;
                Thread thread = new Thread(this.ExeFile);
                thread.Start();
            }
            FormNotify form = new FormNotify();
            int x = Screen.PrimaryScreen.WorkingArea.Width - form.Width - 15;
            int y = Screen.PrimaryScreen.WorkingArea.Height - form.Height;
            form.Location = new System.Drawing.Point(x, y);
            form.TopMost = true;
            form.Show(item);
        }

        //执行文件或程序
        private void ExeFile()
        {
            if (File.Exists(_exeFilePath))
            {
                System.Diagnostics.Process.Start(_exeFilePath);
            }
        }

        //加载当天需要提醒的数据
        private void LoadData()
        {
            _list.Clear();
                
            DateTime now = DateTime.Now;
            string state = PublicEnums.State.未完成.ToString();

            string sql = string.Format("select * from T_CONTENT where STATE = '{0}'",state);
            DataTable dt = _db.ExecuteDataTable(sql);

            NotifyEntity entity = null;
            foreach (DataRow item in dt.Rows)
            {
                Content content = new Content();
                content.Catalog_Id = Convert.ToInt32(item["Catalog_Id"]);
                content.Content_Id = Convert.ToInt32(item["Content_Id"]);
                content.ContentText = item["ContentText"].ToString();
                content.ContentTitle = item["ContentTitle"].ToString();
                content.Exe_Path = item["Exe_Path"].ToString();
                content.Interval_Days = Convert.ToInt32(item["Interval_Days"]);
                content.Is_Exe_Notify = Convert.ToInt32(item["Is_Exe_Notify"]);
                content.State = item["State"].ToString();
                content.Time_Create = Convert.ToDateTime(item["Time_Create"]);
                if (item["Time_Finish"] != DBNull.Value)
                {
                    content.Time_Finish = Convert.ToDateTime(item["Time_Finish"]);
                }
                content.Time_Notify = Convert.ToInt32(item["Time_Notify"]);
                content.Time_Predict_Finish = Convert.ToDateTime(item["Time_Predict_Finish"]);

                if ((now - Convert.ToDateTime(item["Time_Create"])).Days % Convert.ToInt32(item["Interval_Days"]) == 0)
                {
                    entity = new NotifyEntity();
                    entity.NotifyContent = content;
                    entity.NotifyTime = content.Time_Notify;
                    _list.Add(entity);
                }
            }
        }

        #region 公共方法

        /// <summary>
        /// 添加到提醒列表
        /// </summary>
        /// <param name="content">要提醒的内容</param>
        public void Add(Content content)
        {
            NotifyEntity entity = new NotifyEntity();
            entity.NotifyContent = content;
            entity.NotifyTime = content.Time_Notify;
            _list.Add(entity);
        }

        /// <summary>
        /// 从提醒列表中删除
        /// </summary>
        /// <param name="content">要删除提醒内容</param>
        public void Remove(Content content)
        {
            NotifyEntity entity = _list.Find(notify => notify.NotifyContent.Content_Id == content.Content_Id);
            if (entity != null)
            {
                _list.Remove(entity);
            }
        }

        /// <summary>
        /// 更新提醒列表中的内容
        /// </summary>
        /// <param name="content">要提醒的内容</param>
        public void Update(Content content)
        {
            NotifyEntity entity = _list.Find(notify => notify.NotifyContent.Content_Id == content.Content_Id);
            if (entity != null)
            {
                entity.NotifyContent = content;
                entity.NotifyTime = content.Time_Notify;
            }
        }

        /// <summary>
        /// 得到NotifyList的一个实例
        /// </summary>
        /// <returns></returns>
        public static NotifyList GetInstance()
        {
            if (_notifyList == null)
            {
                _notifyList = new NotifyList();
            }
            return _notifyList;
        }

        /// <summary>
        /// 开始扫描需要提醒的数据
        /// </summary>
        public void StartScan()
        {
            _timer.Start();
        }

        /// <summary>
        /// 停止扫描
        /// </summary>
        public void StopScan()
        {
            _timer.Stop();
        }

        /// <summary>
        /// 获取未提醒的数据
        /// </summary>
        /// <returns></returns>
        internal List<Content> GetNotNotifyTask()
        {
            DateTime now = DateTime.Now;
            int notifyTime = now.Hour * 60 + now.Minute;
            List<Content> list = new List<Content>();
            foreach (var item in _list)
            {
                if (item.NotifyTime > notifyTime)
                {
                    list.Add(item.NotifyContent);
                }
            }
            return list;
        }

        #endregion
    }

    /// <summary>
    /// 提醒内容和时间
    /// </summary>
    public class NotifyEntity
    {
        /// <summary>
        /// 任务
        /// </summary>
        public Content NotifyContent { get; set; }

        /// <summary>
        /// 提醒时间
        /// </summary>
        public int NotifyTime { get; set; }
    }
}
