﻿/*
 * author:Joey Zhao
 * date:2010-10-3
 * descirbe：catalog form
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using System.Collections;
using Joey.DbOperator;
using GuYu.Todo.Forms.BusinessCatalog;
using GuYu.Todo.Entities;

namespace GuYu.Todo.Forms
{
    public partial class FormCatalog : DockContent
    {
        private CatalogBusiness _business = new CatalogBusiness();
        private readonly string CON_NEW_NODE_NAME = "新建目录";//新建目录的默认名称
        private TreeNode _currentNode;//当前选择的节点

        public FormCatalog()
        {
            InitializeComponent();
            this.treeView1.AfterSelect += new TreeViewEventHandler(treeView1_AfterSelect);
            this.treeView1.MouseClick += new MouseEventHandler(treeView1_MouseClick);
            this.Load += new EventHandler(FormCatalog_Load);
            this.ToolStripMenuItemNewChildCatalog.Click += new EventHandler(ToolStripMenuItemNewChildCatalog_Click);
            this.ToolStripMenuItemDeleteCatalog.Click += new EventHandler(ToolStripMenuItemDeleteCatalog_Click);
            this.treeView1.AfterLabelEdit += new NodeLabelEditEventHandler(treeView1_AfterLabelEdit);
        }

        //编辑节点
        private void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (e.Label == null)
            {
                return;
            }
            if (e.Label.Trim() == string.Empty)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("目录名称不能为空！");
                e.CancelEdit = true;
                return;
            }
            TreeNode rootNode = e.Node.Parent;
            if (rootNode != null)
            {
                if (this.ExistSameNode(rootNode, e.Label))
                {
                    PublicUnit.ControlsHelper.ShowInforMessage("同一父目录下已经存在相同的子目录！");
                    e.CancelEdit = true;
                    return;
                }
            }

            int catalogId = (int)e.Node.Tag;
            string catalogName = e.Label;
            _business.MondifyCatalogName(catalogId, catalogName);
        }

        //点击删除右键菜单
        private void ToolStripMenuItemDeleteCatalog_Click(object sender, EventArgs e)
        {
            int currentCatalogId = (int)this.treeView1.SelectedNode.Tag;
            if(PublicUnit.ControlsHelper.ShowOkCancelMessage("删除后不能恢复，是否继续","删除确认"))
            {
                _business.DeleteCatalog(currentCatalogId);
                this.treeView1.SelectedNode.Remove();
            }
        }

        //点击新建菜单
        private void ToolStripMenuItemNewChildCatalog_Click(object sender, EventArgs e)
        {
            Catalog newCatalog = new Catalog();
            newCatalog.Par_Id = (int)this.treeView1.SelectedNode.Tag;
            newCatalog.Catalog_Name = this.GetNewNodeName(this.treeView1.SelectedNode);
            _business.SaveCatalog(newCatalog);
            TreeNode newNode = new TreeNode();
            newNode.Text = newCatalog.Catalog_Name;
            newNode.Tag = newCatalog.Catalog_Id;
            this.treeView1.SelectedNode.Nodes.Add(newNode);
            this.treeView1.SelectedNode.Expand();
        }

        //得到参数制定节点下新建子节点的名称
        private string GetNewNodeName(TreeNode treeNode)
        {
            string newNodeName = CON_NEW_NODE_NAME;
            if (!this.ExistSameNode(treeNode,newNodeName))
            {
                return newNodeName;
            }
            int i = 1;
            while (true)
            {
                newNodeName = CON_NEW_NODE_NAME + i.ToString();
                i++;
                if (!this.ExistSameNode(treeNode,newNodeName))
                {
                    return newNodeName;
                }
            }
        }

        //鼠标点击事件(右键点击显示菜单)
        private void treeView1_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Right)
            {
                TreeNode node = this.treeView1.GetNodeAt(e.Location);
                if (node != null)
                {
                    Point p = this.PointToScreen(e.Location);
                    if (node == this.treeView1.Nodes[0])
                    {
                        this.ToolStripMenuItemDeleteCatalog.Enabled = false;
                    }
                    else
                    {
                        this.ToolStripMenuItemDeleteCatalog.Enabled = true;
                    }
                    this.contextMenuStrip1.Show(p);
                }
            }
        }

        //选定树节点后触发
        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (this.CurrentCatalogChanged == null)
            {
                return;
            }
            if (_currentNode != null)
            {
                _currentNode.ForeColor = Color.Black;
            }
            _currentNode = e.Node;
            _currentNode.ForeColor = Color.Red;
            int catalogId = (int)e.Node.Tag;
            CatalogEventArgs eventArgs = new CatalogEventArgs();
            eventArgs.CatalogId = catalogId;
            this.CurrentCatalogChanged(sender, eventArgs);
        }

        //Load事件
        private void FormCatalog_Load(object sender, EventArgs e)
        {
            this.LoadCatalogData();
        }

        //加载目录数据
        private void LoadCatalogData()
        {
            List<Catalog> catalogSet = _business.GetCatalogSet();
            if (catalogSet.Count() < 1)
            {
                Catalog catalogRoot = _business.CreateRootCatalog();
                TreeNode node = new TreeNode();
                node.Text = catalogRoot.Catalog_Name;
                node.Tag = catalogRoot.Catalog_Id;
                this.treeView1.Nodes.Add(node);
            }//end if
            else
            {
                //找到根目录
                Catalog rootCatalog = _business.GetRootCatalog();
                //加载根目录
                TreeNode rootNode = new TreeNode();
                rootNode.Text = rootCatalog.Catalog_Name;
                rootNode.Tag = rootCatalog.Catalog_Id;
                this.treeView1.Nodes.Add(rootNode);
                Queue<TreeNode> nodesQueue = new Queue<TreeNode>();
                nodesQueue.Enqueue(rootNode);
                //加载子目录
                TreeNode currentRootNode;
                TreeNode newChildNode;
                while (nodesQueue.Count > 0)
                {
                    currentRootNode = nodesQueue.Dequeue();
                    var childCatalogSet = from c in catalogSet
                                          where c.Par_Id == (int)currentRootNode.Tag
                                          select c;
                    foreach (var item in childCatalogSet)
                    {
                        newChildNode = new TreeNode();
                        newChildNode.Text = item.Catalog_Name;
                        newChildNode.Tag = item.Catalog_Id;
                        currentRootNode.Nodes.Add(newChildNode);
                        nodesQueue.Enqueue(newChildNode);
                    }
                }
            }// end else
            this.treeView1.SelectedNode = this.treeView1.Nodes[0];
        }

        //判断制定目录下是否已经存在名称为参数指定的节点
        private bool ExistSameNode(TreeNode rootNode,string name)
        {
            foreach (TreeNode item in rootNode.Nodes)
            {
                if (item.Text == name)
                {
                    return true;
                }
            }
            return false;
        }

        #region 公共方法、属性、事件

        /// <summary>
        /// 当前目录改变时触发
        /// </summary>
        public EventHandler<CatalogEventArgs> CurrentCatalogChanged;

        /// <summary>
        /// 获得当前选择的目录ID
        /// </summary>
        public int CurrentCatalogId 
        {
            get
            {
                return (int)this.treeView1.SelectedNode.Tag;
            }
        }

        /// <summary>
        /// 设置目录的Enabled属性
        /// </summary>
        public bool CatalogEnabled
        {
            set { this.treeView1.Enabled = value; }
        }

        #endregion
    }

    public class CatalogEventArgs : EventArgs
    {
        /// <summary>
        /// 目录ID
        /// </summary>
        public int CatalogId { get; set; }
    }
}
