﻿/*
 * author:Joey Zhao
 * date:2010-10-3
 * descirbe:todo list form
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using System.Threading;
using System.Globalization;
using GuYu.Todo.Entities;
using GuYu.Todo.Forms.BusinessContent;
using GuYu.Todo.Forms.BusinessNotify;
using GuYu.Todo.PublicUnit;

namespace GuYu.Todo.Forms
{
    public partial class FormList : DockContent
    {
        private ContentBusiness _business = new ContentBusiness();
        private BindingList<Content> _contentList;
        private readonly string CON_ALL = "全部";
        private int _catalogId;
        private int _currentPageIndex;//当前页
        private readonly int CON_PAGE_COUNT = 17;//每页显示的记录数
        private int _totalCount = 0;//记录总数
        private int _totalPagesCount;//总页数

        public FormList()
        {
            InitializeComponent();
            _contentList = new BindingList<Content>();
            this.comboBoxState.Items.Add(CON_ALL);
            this.comboBoxState.BindComboBoxWithEnum(typeof(PublicEnums.State));
            this.comboBoxState.SelectedIndex = 2;
            this.Load += new EventHandler(FormList_Load);
            this.toolStripButtonNew.Click += new EventHandler(toolStripButtonNew_Click);
            this.toolStripButtonMondify.Click += new EventHandler(toolStripButtonMondify_Click);
            this.toolStripButtonDelete.Click += new EventHandler(toolStripButtonDelete_Click);
            this.toolStripButtonFinish.Click += new EventHandler(toolStripButtonFinish_Click);
            this.toolStripButtonCancel.Click += new EventHandler(toolStripButtonCancel_Click);
            this.buttonQuery.Click += new EventHandler(buttonQuery_Click);
            this.dataGridViewContent.CellDoubleClick += new DataGridViewCellEventHandler(dataGridViewContent_CellDoubleClick);

            this.toolStripButtonFirstPage.Click += delegate(object o, EventArgs e) { this.BindData(1); };
            this.toolStripButtonPrePage.Click += delegate(object o, EventArgs e) { this.BindData(_currentPageIndex - 1); };
            this.toolStripButtonLastPage.Click += delegate(object o, EventArgs e) { this.BindData(_totalCount / CON_PAGE_COUNT + (_totalCount % CON_PAGE_COUNT == 0 ? 0 : 1)); };
            this.toolStripButtonNextPage.Click += delegate(object o, EventArgs e) { this.BindData(_currentPageIndex + 1); };
            this.toolStripTextBoxPage.TextChanged += new EventHandler(toolStripTextBoxPage_TextChanged);
        }

        //加载第一页数据
        private void FormList_Load(object sender, EventArgs e)
        {
            this.BindData(1);
        }

        //双击事件
        private void dataGridViewContent_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex == -1)
            {
                return;
            }
            this.toolStripButtonMondify_Click(null, null);
        }

        //点击查询按钮
        private void buttonQuery_Click(object sender, EventArgs e)
        {
            if (this.dateTimePickerFrom.Value.Date > this.dateTimePickerTo.Value.Date)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("开始时间不能大于结束时间！");
                this.dateTimePickerFrom.Focus();
                return;
            }
            this.BindData(1);
        }

        //点击“标记为未完成”按钮
        private void toolStripButtonCancel_Click(object sender, EventArgs e)
        {
            if (this.CurrentContent == null)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("请先选择要标记为\"未完成\"的数据!");
                return;
            }
            if (this.CurrentContent.State == PublicEnums.State.未完成.ToString())
            {
                PublicUnit.ControlsHelper.ShowInforMessage("所选数据已经为\"未完成\"状态！");
                return;
            }
            else if (this.CurrentContent.State == PublicEnums.State.已过期.ToString())
            {
                PublicUnit.ControlsHelper.ShowInforMessage("已过期的数据不能设置为\"未完成\"状态！");
                return;
            }
            this.CurrentContent.State = PublicEnums.State.未完成.ToString();
            _business.MondifyState(this.CurrentContent.Content_Id, PublicEnums.State.未完成.ToString());
            NotifyList.GetInstance().Add(this.CurrentContent);
            this.BindData(_currentPageIndex);
        }

        //点击“标记为完成”按钮
        private void toolStripButtonFinish_Click(object sender, EventArgs e)
        {
            if (this.CurrentContent == null)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("请先选择要标记为完成的数据!");
                return;
            }
            if (this.CurrentContent.State == PublicEnums.State.完成.ToString())
            {
                PublicUnit.ControlsHelper.ShowInforMessage("所选数据已经为“完成”状态！");
                return;
            }
            else if (this.CurrentContent.State == PublicEnums.State.已过期.ToString())
            {
                PublicUnit.ControlsHelper.ShowInforMessage("已过期的数据不能设置为\"完成\"状态！");
                return;
            }
            this.CurrentContent.State = PublicEnums.State.完成.ToString();
            _business.MondifyState(this.CurrentContent.Content_Id, PublicEnums.State.完成.ToString());
            NotifyList.GetInstance().Remove(this.CurrentContent);
            this.BindData(_currentPageIndex);
        }

        //点击删除按钮
        private void toolStripButtonDelete_Click(object sender, EventArgs e)
        {
            if (this.CurrentContent == null)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("请先选择要删除的数据！");
                return;
            }
            if (PublicUnit.ControlsHelper.ShowOkCancelMessage("确认要删除吗？","删除确认"))
            {
                _business.DeleteContent(this.CurrentContent);
                if (this.CurrentContent.State == PublicEnums.State.未完成.ToString())
                {
                    NotifyList.GetInstance().Remove(this.CurrentContent);
                }
                _contentList.Remove(this.CurrentContent);
            }
        }

        //点击修改按钮
        private void toolStripButtonMondify_Click(object sender, EventArgs e)
        {
            if (this.CurrentContent == null)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("请先选择要修改的数据！");
                return;
            }
            FormEdit form = new FormEdit();
            if (form.ShowDialog(this.CurrentContent, _catalogId, this.CurrentContent.State == PublicEnums.State.完成.ToString() || this.CurrentContent.State== PublicEnums.State.已过期.ToString() ? PublicEnums.EditType.View : PublicEnums.EditType.Mondify) == System.Windows.Forms.DialogResult.OK)
            {
                _business.Update(this.CurrentContent);
                if (this.CurrentContent.State == PublicEnums.State.未完成.ToString())
                {
                    NotifyList.GetInstance().Update(this.CurrentContent);
                }
            }
        }

        //点击新建按钮
        private void toolStripButtonNew_Click(object sender, EventArgs e)
        {
            FormEdit form = new FormEdit();
            if (form.ShowDialog(null, _catalogId,PublicEnums.EditType.New) == System.Windows.Forms.DialogResult.OK)
            {
                Content newContent = form.Content;
                newContent.Catalog_Id = _catalogId;
                newContent.State = PublicEnums.State.未完成.ToString();
                _business.Save(newContent);
                _contentList.Add(newContent);
                this.dataGridViewContent.DataSource = _contentList;
                NotifyList.GetInstance().Add(newContent);
            }
        }

        //当前选择的对象
        private Content CurrentContent
        {
            get
            {
                if (this.dataGridViewContent.SelectedRows.Count < 1)
                {
                    return null;
                }
                int contentId = Convert.ToInt32(this.dataGridViewContent.SelectedRows[0].Cells[this.ColumnId.Name].Value);
                return _contentList.First(c => c.Content_Id == contentId);
            }
        }

        // 绑定数据
        private void BindData(int pageIndex)
        {
            DateTime toDateTime = this.dateTimePickerTo.Value.Date.AddDays(1d).AddSeconds(-1d);
            DateTime fromDateTime = this.dateTimePickerFrom.Value.Date;
            int catalogId = _catalogId;
            string contentTitle = this.textBoxTitle.Text.Trim();
            string state = this.comboBoxState.Text;
            int fromIndex = (pageIndex - 1) * CON_PAGE_COUNT;
            int count = CON_PAGE_COUNT;

            _contentList = _business.GetContent(fromDateTime, toDateTime, catalogId, contentTitle, state, fromIndex, count, out _totalCount);

            if (_totalCount == 0)
            {
                _currentPageIndex = 0;
                _totalPagesCount = 0;
            }
            else
            {
                _currentPageIndex = pageIndex;
                _totalPagesCount = _totalCount / CON_PAGE_COUNT + (_totalCount % CON_PAGE_COUNT == 0 ? 0 : 1);
            }
          
            this.toolStripLabelTotalPageCount.Text = "/" + _totalPagesCount.ToString();
            this.toolStripTextBoxPage.Text = _currentPageIndex.ToString();
            if (_currentPageIndex == _totalPagesCount)
            {
                this.toolStripButtonNextPage.Enabled = false;
                this.toolStripButtonLastPage.Enabled = false;
            }
            else
            {
                this.toolStripButtonNextPage.Enabled = true;
                this.toolStripButtonLastPage.Enabled = true;
            }
            if (_currentPageIndex == 1)
            {
                this.toolStripButtonFirstPage.Enabled = false;
                this.toolStripButtonPrePage.Enabled = false;
            }
            else if (_currentPageIndex == 0)
            {
                this.toolStripButtonFirstPage.Enabled = false;
                this.toolStripButtonPrePage.Enabled = false;
                this.toolStripButtonFirstPage.Enabled = false;
                this.toolStripButtonPrePage.Enabled = false;
            }
            else
            {
                this.toolStripButtonFirstPage.Enabled = true;
                this.toolStripButtonPrePage.Enabled = true;
            }

            this.dataGridViewContent.DataSource = _contentList;
        }

        //分页相关--输入页数
        private void toolStripTextBoxPage_TextChanged(object sender, EventArgs e)
        {
            int index;
            if (Int32.TryParse(this.toolStripTextBoxPage.Text.Trim(), out index))
            {
                if (index != _currentPageIndex && index > 0 && index <= _totalPagesCount)
                {
                    this.BindData(index);
                }
            }
        }

        #region 公共方法、属性

        /// <summary>
        /// 设置目录ID
        /// </summary>
        public int CatalogId
        {
            set
            {
                _catalogId = value;
            }
        }

        /// <summary>
        /// 加载数据
        /// </summary>
        public void BindData()
        {
            this.BindData(1);
        }

        #endregion
    }
}
