﻿/*
 * autor:Joey
 * date:2011-1-10 
 * describe:数据库操作类
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Common;
using System.Data;

using Joey.DbOperator.ADODotNetFactory;
using System.Reflection;

namespace Joey.DbOperator
{
    public class DataBase
    {
        private DbConnection _con;
        private DbTransaction _tran;
        private AbsADODotNet _adoDotNet;
        private AbsADODotNetFactory _adoDotNetFactory;

        /// <summary>
        /// 根据配置文件(app.config)初始化连接字符串
        /// </summary>
        /// <exception cref="Exception"></exception>
        public DataBase()
        {
            try
            {
                string factroyName = System.Configuration.ConfigurationManager.AppSettings["ADODotNetFactory"];
                _adoDotNetFactory = (AbsADODotNetFactory)Assembly.Load("Joey.DbOperator").CreateInstance(factroyName);
                _adoDotNet = _adoDotNetFactory.Create();

                _con = _adoDotNet.GetConnection();
                _con.ConnectionString = System.Configuration.ConfigurationManager.AppSettings["ConnectionString"];
            }
            catch (Exception ex)
            {
                throw new Exception("创建连接出错: " + ex.Message);
            }
        }

        /// <summary>
        /// 根据配置文件初始化连接字符串
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="adoDotNetFactory">工厂类名称</param>
        /// <param name="connectionString">连接字符串</param>
        public DataBase(string connectionString, string adoDotNetFactory)
        {
            try
            {
                string factroyName = adoDotNetFactory;
                _adoDotNetFactory = (AbsADODotNetFactory)Assembly.Load("Joey.DbOperator").CreateInstance(factroyName);
                _adoDotNet = _adoDotNetFactory.Create();

                _con = _adoDotNet.GetConnection();
                _con.ConnectionString = connectionString;
            }
            catch (Exception ex)
            {
                throw new Exception("创建连接出错: " + ex.Message);
            }
        }

        //创建DbCommand对象
        private DbCommand BuildCommand(string commandText, CommandType commandType, DbParameter[] paramArray)
        {
            DbCommand cmd = _adoDotNet.GetCommand(); 
            cmd.CommandText = commandText;
            cmd.CommandType = commandType;
            cmd.Connection = _con;
            cmd.Parameters.AddRange(paramArray);
            return cmd;
        }

        /// <summary>
        /// 得到指定表的全部数据
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="tableName">表名称</param>
        /// <returns>数据集</returns>
        public DataSet GetEntity(string tableName)
        {
            try
            {
                #region 不好用呀，不好用
                //DbProviderFactory factory = DbProviderFactories.GetFactory("System.Data.SQLite");
                //DbDataAdapter adapter = factory.CreateDataAdapter();
                //string sql = string.Format(@"select * from {0}", tableName);
                //DbCommand cmd = factory.CreateCommand();
                //DbConnection con = factory.CreateConnection();
                //con.ConnectionString = "data source= db\\TestDb.db3;";
                //cmd.CommandText = sql;
                //cmd.Connection = con;
                //adapter.SelectCommand = cmd;

                //DataSet ds = new DataSet();
                //con.Open();
                //adapter.Fill(ds, tableName);
                //return ds;
                #endregion

                DbDataAdapter adapter = _adoDotNet.GetDataAdapter();
                string sql = string.Format(@"select * from {0}", tableName);
                DbCommand cmd = _adoDotNet.GetCommand();
                cmd.CommandText = sql;
                cmd.Connection = _con;
                adapter.SelectCommand = cmd;

                DataSet ds = new DataSet();
                _con.Open();
                adapter.Fill(ds, tableName);
                return ds;
            }
            catch (Exception ex)
            {
                throw new Exception("获取数据出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行返回数据集的SQL
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">sql语句</param>
        /// <param name="paramArray">参数（可选）</param>
        /// <returns>结果集</returns>
        public DataTable ExecuteDataTable(string sql, params DbParameter[] paramArray)
        {
            try
            {
                DbDataAdapter adapter = _adoDotNet.GetDataAdapter();
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                adapter.SelectCommand = cmd;

                DataTable dt = new DataTable();
                _con.Open();
                adapter.Fill(dt);
                return dt;
            }
            catch (Exception ex)
            {
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行返回数据集的SQL(用于分页)
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">Sql语句</param>
        /// <param name="from">起始位置</param>
        /// <param name="count">返回的行数</param>
        /// <param name="paramArray">参数（可选）</param>
        /// <returns>结果集</returns>
        public DataTable ExecuteDataTable(string sql, int from, int count, params DbParameter[] paramArray)
        {
            try
            {
                DbDataAdapter adapter = _adoDotNet.GetDataAdapter();
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);

                DataTable dt = new DataTable();
                adapter.SelectCommand = cmd;
                _con.Open();
                adapter.Fill(from, count, dt);
                return dt;
            }
            catch (Exception ex)
            {
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行返回数据集的SQL
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">sql语句</param>
        /// <param name="paramArray">参数（可选）</param>
        /// <returns>结果集</returns>
        public DataSet ExecuteDataSet(string sql, params DbParameter[] paramArray)
        {
            try
            {
                DbDataAdapter adapter = _adoDotNet.GetDataAdapter();
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                adapter.SelectCommand = cmd;

                DataSet ds = new DataSet();
                _con.Open();
                adapter.Fill(ds);
                return ds;
            }
            catch (Exception ex)
            {
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行返回数据集的存储过程(该方法还没有测试)
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="procedureName">存储过程名称</param>
        /// <param name="paramArray">参数（可选）</param>
        /// <returns>结果集</returns>
        public DataSet ExecuteDataSetProce(string procedureName, params DbParameter[] paramArray)
        {
            try
            {
                DbDataAdapter adapter = _adoDotNet.GetDataAdapter();
                DbCommand cmd = BuildCommand(procedureName, CommandType.StoredProcedure, paramArray);
                adapter.SelectCommand = cmd;

                DataSet ds = new DataSet();
                _con.Open();
                adapter.Fill(ds);
                return ds;
            }
            catch (Exception ex)
            {
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行返回数据集的SQL(用于分页)
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">Sql语句</param>
        /// <param name="from">起始位置</param>
        /// <param name="count">返回的行数</param>
        /// <param name="paramArray">参数（可选）</param>
        /// <returns>结果集</returns>
        public DataSet ExecuteDataSet(string sql, int from, int count, params DbParameter[] paramArray)
        {
            try
            {
                DbDataAdapter adapter = _adoDotNet.GetDataAdapter();
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                adapter.SelectCommand = cmd;

                DataSet ds = new DataSet();
                _con.Open();
                adapter.Fill(ds, from, count, "temptable");
                return ds;
            }
            catch (Exception ex)
            {
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 保存DataTable
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="dt">datatable</param>
        /// <returns>影响的行数</returns>
        public int SaveDataTable(DataTable dt)
        {
            try
            {
                DbDataAdapter adapter = _adoDotNet.GetDataAdapter();
                string sql = string.Format(@"select * from {0}", dt.TableName);
                DbCommand cmd = _adoDotNet.GetCommand();
                DbCommandBuilder comBuilder = _adoDotNet.GetCommandBuilder();

                comBuilder.DataAdapter = adapter;
                cmd.CommandText = sql;
                cmd.Connection = _con;
                adapter.SelectCommand = cmd;
                adapter.InsertCommand = comBuilder.GetInsertCommand();
                adapter.UpdateCommand = comBuilder.GetUpdateCommand();
                adapter.DeleteCommand = comBuilder.GetDeleteCommand();

                _con.Open();
                _tran = _con.BeginTransaction();//transaction begin 传说中sqlite每执行一条insert语句都开启一个事务，死慢
                cmd.Transaction = _tran;
                int result = adapter.Update(dt);
                _tran.Commit();//transaction end
                return result;
            }
            catch (Exception ex)
            {
                _tran.Rollback();
                throw new Exception("保存DataTable出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 保存DataSet
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="ds">要保存的DataSet</param>
        /// <returns>影响的行数</returns>
        public int SaveDataSet(DataSet ds)
        {
            try
            {
                int result = 0;
                foreach (DataTable item in ds.Tables)
                {
                    result += this.SaveDataTable(item);
                }
                return result;
            }
            catch (Exception ex)
            {
                throw new Exception("保存DataSet出错：" + ex.Message);
            }
        }

        /// <summary>
        /// 执行返回一个结果的SQL
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">SQL语句</param>
        /// <param name="paramArray">参数（可选）</param>
        /// <returns>执行结果</returns>
        public Object ExecuteScalar(string sql, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                _con.Open();
                Object obj = cmd.ExecuteScalar();
                return obj;
            }
            catch (Exception ex)
            {
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行返回一个结果的SQL(在一个事务中)
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">SQL语句</param>
        /// <param name="paramArray">参数（可选）</param>
        /// <returns>执行结果</returns>
        public Object ExecuteScalarInTrans(string sql, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                _con.Open();
                _tran = _con.BeginTransaction();
                cmd.Transaction = _tran;
                Object obj = cmd.ExecuteScalar();
                _tran.Commit();
                return obj;
            }
            catch (Exception ex)
            {
                _tran.Rollback();
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行返回单一结果的存储过程
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="procedureName">存储过程名称</param>
        /// <param name="paramArray">存储过程参数</param>
        public object ExecuteScalarProce(string procedureName, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(procedureName, CommandType.StoredProcedure, paramArray);
                _con.Open();
                return cmd.ExecuteScalar();
            }
            catch (Exception ex)
            {
                throw new Exception("执行存储过程出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行无返回结果的SQL
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">语句</param>
        /// <param name="parameter">参数（可选）</param>
        /// <returns>影响的行数</returns>
        public int ExecuteNonQuery(string sql, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                _con.Open();
                return cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行无返回结果的SQL(在一个事务中)
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">语句</param>
        /// <param name="parameter">参数（可选）</param>
        /// <returns>影响的行数</returns>
        public int ExecuteNonQueryInTrans(string sql, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                _con.Open();
                _tran = _con.BeginTransaction();
                cmd.Transaction = _tran;
                int result = cmd.ExecuteNonQuery();
                _tran.Commit();
                return result;
            }
            catch (Exception ex)
            {
                _tran.Rollback();
                throw new Exception("执行SQL语句出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 执行无返回结果的存储过程
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="procedureName">存储过程名称</param>
        /// <param name="paramArray">存储过程参数</param>
        /// <returns>影响的行数</returns>
        public int ExecuteNonQueryProce(string procedureName, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(procedureName, CommandType.StoredProcedure, paramArray);
                _con.Open();
                return cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw new Exception("执行存储过程出错：" + ex.Message);
            }
            finally
            {
                _con.Close();
            }
        }

        /// <summary>
        /// 根据SQl语句得到一个IDataReader对象
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">Sql语句</param>
        /// <returns>IDataReader对象</returns>
        public IDataReader ExecuteReader(string sql, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(sql, CommandType.Text, paramArray);
                _con.Open();
                return cmd.ExecuteReader();
            }
            catch (Exception ex)
            {
                _con.Close();
                throw new Exception("执行SQL语句出错:" + ex.Message);
            }
        }

        /// <summary>
        /// 执行一个存储过程得到IDataReader对象(该方法还没有测试)
        /// </summary>
        /// <exception cref="Exception"></exception>
        /// <param name="sql">存储过程名称</param>
        /// <returns>IDataReader对象</returns>
        public IDataReader ExecuteReaderProce(string procedureName, params DbParameter[] paramArray)
        {
            try
            {
                DbCommand cmd = this.BuildCommand(procedureName, CommandType.StoredProcedure, paramArray);
                _con.Open();
                return cmd.ExecuteReader();
            }
            catch (Exception ex)
            {
                _con.Close();
                throw new Exception("执行SQL语句出错:" + ex.Message);
            }
        }

        /// <summary>
        /// 关闭连接
        /// </summary>
        /// <exception cref="Exception"></exception>
        public void CloseConnection()
        {
            try
            {
                if (_con.State != ConnectionState.Closed)
                {
                    _con.Close();
                }
            }
            catch (Exception ex)
            {
                throw new Exception("关闭连接出错：" + ex.Message);
            }
        }

        /// <summary>
        /// 获得一个sql参数对象
        /// </summary>
        /// <param name="name">名称</param>
        /// <returns>DbParameter实例</returns>
        public DbParameter GetParameter()
        {
            return _adoDotNet.GetParameter();
        }

        /// <summary>
        /// 获取SQL参数前缀
        /// </summary>
        /// <returns>SQL参数前缀</returns>
        public string GetParameterPrefix()
        {
            return _adoDotNet.GetParameterPrefix();
        }
    }
}

